window.Gleam.initSearch([{"doc":"mug","title":"mug","content":"# mug 🍺\n\n[![Package Version](https://img.shields.io/hexpm/v/mug)](https://hex.pm/packages/mug)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/mug/)\n\nA TCP client for Gleam!\n\n```sh\ngleam add mug\n```\n\n```gleam\nimport mug\n\npub fn main() {\n  // Form a connection to a TCP server\n  let assert Ok(socket) =\n    mug.new(&quot;erlang-the-movie.example.com&quot;, port: 12345)\n    |&gt; mug.timeout(milliseconds: 500)\n    |&gt; mug.connect()\n\n  // Send a packet to the server\n  let assert Ok(Nil) = mug.send(socket, &lt;&lt;&quot;Hello, Joe!\\n&quot;:utf8&gt;&gt;)\n\n  // Receive a packet back\n  let assert Ok(packet) = mug.receive(socket, timeout_milliseconds: 100)\n  \n  packet\n  // -&gt; &lt;&lt;&quot;Hello, Mike!&quot;:utf8&gt;&gt;\n}\n```\n\nIt also includes support for receiving packages as Erlang messages, enabling\nTCP sockets to be used within OTP actors.\n\nDocumentation can be found at &lt;https://hexdocs.pm/mug&gt;.\n","url":"index.html"},{"doc":"mug","title":"ConnectionOptions","content":"pub type ConnectionOptions {\n  ConnectionOptions(host: String, port: Int, timeout: Int)\n}\n\nConnectionOptions(host: String, port: Int, timeout: Int)\n\nhost\n&lt;p&gt;The hostname of the server to connect to.&lt;/p&gt;\n\nport\n&lt;p&gt;The port of the server to connect to.&lt;/p&gt;\n\ntimeout\n&lt;p&gt;A timeout in milliseconds for the connection to be established.&lt;/p&gt;\n&lt;p&gt;Note that if the operating system returns a timeout then this package\nwill also return a timeout, even if this timeout value has not been\nreached yet.&lt;/p&gt;\n\nSynonyms:\nmug.ConnectionOptions\nmug ConnectionOptions","url":"mug.html#ConnectionOptions"},{"doc":"mug","title":"Error","content":"pub type Error {\n  Closed\n  Timeout\n  Eaddrinuse\n  Eaddrnotavail\n  Eafnosupport\n  Ealready\n  Econnaborted\n  Econnrefused\n  Econnreset\n  Edestaddrreq\n  Ehostdown\n  Ehostunreach\n  Einprogress\n  Eisconn\n  Emsgsize\n  Enetdown\n  Enetunreach\n  Enopkg\n  Enoprotoopt\n  Enotconn\n  Enotty\n  Enotsock\n  Eproto\n  Eprotonosupport\n  Eprototype\n  Esocktnosupport\n  Etimedout\n  Ewouldblock\n  Exbadport\n  Exbadseq\n  Nxdomain\n  Eacces\n  Eagain\n  Ebadf\n  Ebadmsg\n  Ebusy\n  Edeadlk\n  Edeadlock\n  Edquot\n  Eexist\n  Efault\n  Efbig\n  Eftype\n  Eintr\n  Einval\n  Eio\n  Eisdir\n  Eloop\n  Emfile\n  Emlink\n  Emultihop\n  Enametoolong\n  Enfile\n  Enobufs\n  Enodev\n  Enolck\n  Enolink\n  Enoent\n  Enomem\n  Enospc\n  Enosr\n  Enostr\n  Enosys\n  Enotblk\n  Enotdir\n  Enotsup\n  Enxio\n  Eopnotsupp\n  Eoverflow\n  Eperm\n  Epipe\n  Erange\n  Erofs\n  Espipe\n  Esrch\n  Estale\n  Etxtbsy\n  Exdev\n}\n Errors that can occur when working with TCP sockets.\n\n For more information on these errors see the Erlang documentation:\n - https://www.erlang.org/doc/man/file#type-posix\n - https://www.erlang.org/doc/man/inet#type-posix\n\nClosed\n\n\nTimeout\n\n\nEaddrinuse\n\n\nEaddrnotavail\n\n\nEafnosupport\n\n\nEalready\n\n\nEconnaborted\n\n\nEconnrefused\n\n\nEconnreset\n\n\nEdestaddrreq\n\n\nEhostdown\n\n\nEhostunreach\n\n\nEinprogress\n\n\nEisconn\n\n\nEmsgsize\n\n\nEnetdown\n\n\nEnetunreach\n\n\nEnopkg\n\n\nEnoprotoopt\n\n\nEnotconn\n\n\nEnotty\n\n\nEnotsock\n\n\nEproto\n\n\nEprotonosupport\n\n\nEprototype\n\n\nEsocktnosupport\n\n\nEtimedout\n\n\nEwouldblock\n\n\nExbadport\n\n\nExbadseq\n\n\nNxdomain\n\n\nEacces\n\n\nEagain\n\n\nEbadf\n\n\nEbadmsg\n\n\nEbusy\n\n\nEdeadlk\n\n\nEdeadlock\n\n\nEdquot\n\n\nEexist\n\n\nEfault\n\n\nEfbig\n\n\nEftype\n\n\nEintr\n\n\nEinval\n\n\nEio\n\n\nEisdir\n\n\nEloop\n\n\nEmfile\n\n\nEmlink\n\n\nEmultihop\n\n\nEnametoolong\n\n\nEnfile\n\n\nEnobufs\n\n\nEnodev\n\n\nEnolck\n\n\nEnolink\n\n\nEnoent\n\n\nEnomem\n\n\nEnospc\n\n\nEnosr\n\n\nEnostr\n\n\nEnosys\n\n\nEnotblk\n\n\nEnotdir\n\n\nEnotsup\n\n\nEnxio\n\n\nEopnotsupp\n\n\nEoverflow\n\n\nEperm\n\n\nEpipe\n\n\nErange\n\n\nErofs\n\n\nEspipe\n\n\nEsrch\n\n\nEstale\n\n\nEtxtbsy\n\n\nExdev\n\n\nSynonyms:\nmug.Error\nmug Error","url":"mug.html#Error"},{"doc":"mug","title":"Socket","content":"pub type Socket\n\n\nSynonyms:\nmug.Socket\nmug Socket","url":"mug.html#Socket"},{"doc":"mug","title":"TcpMessage","content":"pub type TcpMessage {\n  Packet(Socket, BitArray)\n  SocketClosed(Socket)\n  TcpError(Socket, Error)\n}\n Messages that can be sent by the socket to the process that controls it.\n\nPacket(Socket, BitArray)\n A packet has been received from the client.\n\nSocketClosed(Socket)\n The socket has been closed by the client.\n\nTcpError(Socket, Error)\n An error has occurred on the socket.\n\nSynonyms:\nmug.TcpMessage\nmug TcpMessage","url":"mug.html#TcpMessage"},{"doc":"mug","title":"connect","content":"pub fn connect(\n  options: ConnectionOptions,\n) -&gt; Result(Socket, Error)\n Establish a TCP connection to the server specified in the connection\n options.\n\n Returns an error if the connection could not be established.\n\n The socket is created in passive mode, meaning the the `receive` function is\n to be called to receive packets from the client. The\n `receive_next_packet_as_message` function can be used to switch the socket\n to active mode and receive the next packet as an Erlang message.\n\nSynonyms:\nmug.connect\nmug connect","url":"mug.html#connect"},{"doc":"mug","title":"new","content":"pub fn new(host: String, port port: Int) -&gt; ConnectionOptions\n Create a new set of connection options.\n\nSynonyms:\nmug.new\nmug new","url":"mug.html#new"},{"doc":"mug","title":"receive","content":"pub fn receive(\n  socket: Socket,\n  timeout_milliseconds timeout: Int,\n) -&gt; Result(BitArray, Error)\n Receive a packet from the client.\n\n Errors if the socket is closed, if the timeout is reached, or if there is\n some other problem receiving the packet.\n\nSynonyms:\nmug.receive\nmug receive","url":"mug.html#receive"},{"doc":"mug","title":"receive_exact","content":"pub fn receive_exact(\n  socket: Socket,\n  byte_size size: Int,\n  timeout_milliseconds timeout: Int,\n) -&gt; Result(BitArray, Error)\n Receive the specified number of bytes from the client, unless the socket\n was closed, from the other side. In that case, the last read may return\n less bytes.\n If the specified number of bytes is not available to read from the socket\n then the function will block until the bytes are available, or until the\n timeout is reached.\n This directly calls the underlying Erlang function `gen_tcp:recv/3`.\n\n Errors if the socket is closed, if the timeout is reached, or if there is\n some other problem receiving the packet.\nSynonyms:\nmug.receive_exact\nmug receive_exact","url":"mug.html#receive_exact"},{"doc":"mug","title":"receive_next_packet_as_message","content":"pub fn receive_next_packet_as_message(socket: Socket) -&gt; Nil\n Switch the socket to active mode, meaning that the next packet received on\n the socket will be sent as an Erlang message to the socket owner&#39;s inbox.\n\n This is useful for when you wish to have an OTP actor handle incoming\n messages as using the `receive` function would result in the actor being\n blocked and unable to handle other messages while waiting for the next\n packet.\n\n Messages will be send to the process that controls the socket, which is the\n process that established the socket with the `connect` function.\n\nSynonyms:\nmug.receive_next_packet_as_message\nmug receive_next_packet_as_message","url":"mug.html#receive_next_packet_as_message"},{"doc":"mug","title":"selecting_tcp_messages","content":"pub fn selecting_tcp_messages(\n  selector: Selector(a),\n  mapper: fn(TcpMessage) -&gt; a,\n) -&gt; Selector(a)\n Configure a selector to receive messages from TCP sockets.\n\n Note this will receive messages from all TCP sockets that the process\n controls, rather than any specific one. If you wish to only handle messages\n from one socket then use one process per socket.\n\nSynonyms:\nmug.selecting_tcp_messages\nmug selecting_tcp_messages","url":"mug.html#selecting_tcp_messages"},{"doc":"mug","title":"send","content":"pub fn send(\n  socket: Socket,\n  packet: BitArray,\n) -&gt; Result(Nil, Error)\n Send a packet to the client.\n\nSynonyms:\nmug.send\nmug send","url":"mug.html#send"},{"doc":"mug","title":"send_builder","content":"pub fn send_builder(\n  socket: Socket,\n  packet: BytesBuilder,\n) -&gt; Result(Nil, Error)\n Send a packet to the client, the data in `BytesBuilder`. Using this function\n is more efficient turning an `BytesBuilder` or a `StringBuilder` into a\n `BitArray` to use with the `send` function.\n\nSynonyms:\nmug.send_builder\nmug send_builder","url":"mug.html#send_builder"},{"doc":"mug","title":"shutdown","content":"pub fn shutdown(socket: Socket) -&gt; Result(Nil, Error)\n Close the socket, ensuring that any data buffered in the socket is flushed to the operating system kernel socket first.\n\nSynonyms:\nmug.shutdown\nmug shutdown","url":"mug.html#shutdown"},{"doc":"mug","title":"timeout","content":"pub fn timeout(\n  options: ConnectionOptions,\n  milliseconds timeout: Int,\n) -&gt; ConnectionOptions\n Specify a timeout for the connection to be established.\n\nSynonyms:\nmug.timeout\nmug timeout","url":"mug.html#timeout"},{"doc":"mug","title":"mug","content":"","url":"mug.html"}]);